@@ -73,6 +73,7 @@
       (fset 'viper-overlay-p (symbol-function 'extentp))
       (fset 'viper-overlay-get (symbol-function 'extent-property))
       (fset 'viper-move-overlay (symbol-function 'set-extent-endpoints))
+      (fset 'viper-overlay-live-p (symbol-function 'extent-live-p))
       (if (viper-window-display-p)
 	  (fset 'viper-iconify (symbol-function 'iconify-frame)))
       (cond ((viper-has-face-support-p)
@@ -88,6 +89,7 @@
   (fset 'viper-overlay-p (symbol-function 'overlayp))
   (fset 'viper-overlay-get (symbol-function 'overlay-get))
   (fset 'viper-move-overlay (symbol-function 'move-overlay))
+  (fset 'viper-overlay-live-p (symbol-function 'overlayp))
   (if (viper-window-display-p)
       (fset 'viper-iconify (symbol-function 'iconify-or-deiconify-frame)))
   (cond ((viper-has-face-support-p)
@@ -125,20 +127,48 @@
       (modify-frame-parameters
        (selected-frame) (list (cons 'cursor-color new-color)))))
 	 
-(defun viper-save-cursor-color ()
+;; By default, saves current frame cursor color in the
+;; viper-saved-cursor-color-in-replace-mode property of viper-replace-overlay
+(defun viper-save-cursor-color (before-which-mode)
   (if (and (viper-window-display-p) (viper-color-display-p))
       (let ((color (viper-get-cursor-color)))
 	(if (and (stringp color) (viper-color-defined-p color)
 		 (not (string= color viper-replace-overlay-cursor-color)))
-	    (viper-overlay-put viper-replace-overlay 'viper-cursor-color color)))))
+	    (modify-frame-parameters
+	     (selected-frame)
+	     (list
+	      (cons
+	       (if (eq before-which-mode 'before-replace-mode)
+		   'viper-saved-cursor-color-in-replace-mode
+		 'viper-saved-cursor-color-in-insert-mode)
+	       color)))
+	  ))))
 	
-;; restore cursor color from replace overlay
-(defsubst viper-restore-cursor-color-after-replace ()
-  (viper-change-cursor-color
-   (viper-overlay-get viper-replace-overlay 'viper-cursor-color)))
-(defsubst viper-restore-cursor-color-after-insert ()
-  (viper-change-cursor-color viper-saved-cursor-color))
+
+(defsubst viper-get-saved-cursor-color-in-replace-mode ()
+  (or
+   (funcall
+    (if viper-emacs-p 'frame-parameter 'frame-property)
+    (selected-frame)
+    'viper-saved-cursor-color-in-replace-mode)
+   viper-vi-state-cursor-color))
+
+(defsubst viper-get-saved-cursor-color-in-insert-mode ()
+  (or
+   (funcall
+    (if viper-emacs-p 'frame-parameter 'frame-property)
+    (selected-frame)
+    'viper-saved-cursor-color-in-insert-mode)
+   viper-vi-state-cursor-color))
 	 
+;; restore cursor color from replace overlay
+(defun viper-restore-cursor-color(after-which-mode)
+  (if (viper-overlay-p viper-replace-overlay)
+      (viper-change-cursor-color
+       (if (eq after-which-mode 'after-replace-mode)
+	   (viper-get-saved-cursor-color-in-replace-mode)
+	 (viper-get-saved-cursor-color-in-insert-mode))
+       )))
    
 
 ;; Check the current version against the major and minor version numbers
@@ -629,7 +659,8 @@
 
 (defsubst viper-file-checked-in-p (file)
   (and (featurep 'vc-hooks)
-       (vc-backend file)
+       ;; CVS files are considered not checked in
+       (not (memq (vc-backend file) '(nil CVS)))
        (not (vc-locking-user file))))
 
 ;; checkout if visited file is checked in
@@ -649,25 +680,31 @@
     
 
 ;;; Overlays
+(defun viper-put-on-search-overlay (beg end)
+  (if (viper-overlay-p viper-search-overlay)
+      (viper-move-overlay viper-search-overlay beg end)
+    (setq viper-search-overlay (viper-make-overlay beg end (current-buffer)))
+    (viper-overlay-put
+     viper-search-overlay 'priority viper-search-overlay-priority))
+  (viper-overlay-put viper-search-overlay 'face viper-search-face))
 
 ;; Search
 
 (defun viper-flash-search-pattern ()
-  (if (viper-overlay-p viper-search-overlay)
-      (viper-move-overlay
-       viper-search-overlay (match-beginning 0) (match-end 0))
-    (setq viper-search-overlay
-	  (viper-make-overlay
-	   (match-beginning 0) (match-end 0) (current-buffer))))
-  
-  (viper-overlay-put
-   viper-search-overlay 'priority viper-search-overlay-priority)
-  (if (viper-has-face-support-p)
-      (progn
-	(viper-overlay-put viper-search-overlay 'face viper-search-face)
-	(sit-for 2)
-	(viper-overlay-put viper-search-overlay 'face nil))))
+  (if (not (viper-has-face-support-p))
+      nil
+    (viper-put-on-search-overlay (match-beginning 0) (match-end 0))
+    (sit-for 2)
+    (viper-overlay-put viper-search-overlay 'face nil)))
 
+(defun viper-hide-search-overlay ()
+  (if (not (viper-overlay-p viper-search-overlay))
+      (progn
+	(setq viper-search-overlay
+	      (viper-make-overlay beg end (current-buffer)))
+	(viper-overlay-put
+	 viper-search-overlay 'priority viper-search-overlay-priority)))
+  (viper-overlay-put viper-search-overlay 'face nil))
 
 ;; Replace state
 
@@ -675,7 +712,7 @@
   (viper-move-overlay viper-replace-overlay beg end))
   
 (defun viper-set-replace-overlay (beg end)
-  (if (viper-overlay-p viper-replace-overlay)
+  (if (viper-overlay-live-p viper-replace-overlay)
       (viper-move-replace-overlay beg end)
     (setq viper-replace-overlay (viper-make-overlay beg end (current-buffer)))
     ;; never detach
@@ -694,12 +731,14 @@
   (if (viper-has-face-support-p)
       (viper-overlay-put
        viper-replace-overlay 'face viper-replace-overlay-face))
-  (viper-save-cursor-color)
+  (viper-save-cursor-color 'before-replace-mode)
   (viper-change-cursor-color viper-replace-overlay-cursor-color)
   )
   
       
 (defun viper-set-replace-overlay-glyphs (before-glyph after-glyph)
+  (or (viper-overlay-live-p viper-replace-overlay)
+      (viper-set-replace-overlay (point-min) (point-min)))
   (if (or (not (viper-has-face-support-p))
 	  viper-use-replace-region-delimiters)
       (let ((before-name (if viper-xemacs-p 'begin-glyph 'before-string))
@@ -709,8 +748,8 @@
   
 (defun viper-hide-replace-overlay ()
   (viper-set-replace-overlay-glyphs nil nil)
-  (viper-restore-cursor-color-after-replace)
-  (viper-restore-cursor-color-after-insert)
+  (viper-restore-cursor-color 'after-replace-mode)
+  (viper-restore-cursor-color 'after-insert-mode)
   (if (viper-has-face-support-p)
       (viper-overlay-put viper-replace-overlay 'face nil)))
 
